'''
This script was made by neokeee 2022.
Easier Layout of Discord rich presence for API.JSON.
Author: @neokeee & @etherlesss
Last Update: 21/03/22
'''

from datetime import datetime
import requests as r
import psutil, time, os, platform, subprocess

http_errors = {
    "error-response": "<Response [500]>",
    "badgateway-response": "<Response [502]>",
    "failed-response": "<Response [400]>",
    "rejected-response": "<Response [403]>",
    "unauthorize-response": "<Response [401]>",
    "notfound-response": "<Response [404]>",
    "timeout-response": "<Response [408]>",
}

def __http_response__(string, dict):
    '''
    Kill program, http error response.
    '''
    print(string, dict)
    exit()

# req for mac m1
def isMac():
    return platform.system() == "Darwin"

def getProcessorMac():
    return subprocess.check_output(['sysctl','-n','machdep.cpu.brand_string']).decode('utf-8')

def isM1():
    return getProcessorMac() == "Apple M1\n"

class RPCLayout():
    def _isM1():
        return getProcessorMac() == "Apple M1\n"
    def _system(self):
        '''
        Return the operating system
        Darwin: MACOSX
        '''
        if platform.system() == "Windows":
            return "Windows"
        elif platform.system() == "Darwin":
            print(isM1())
            return "Darwin"
        
    def _getpid(self):
        '''
        Returns the program PID, use it for Presence.update(pid=_getpid())
        '''
        return os.getpid()
    def timestamp(self):
        '''
        TimeStamp made by etherlesss.
        '''
        now = datetime.utcnow()
        elapsed = int((now - datetime(1970, 1, 1)).total_seconds())
        return elapsed
    def _detectGame(self, gameExe):
        '''
        :param str gameExe: Game or program (MacOS supported!)
        Find the program.exe and returns True if the function found it, return False if function don't found it
        '''
        game = gameExe in (p.name() for p in psutil.process_iter())
        return game
    def _waitingForOpen(self, game = str):
        '''
        :param str game: game or program. Recieves a string.
    
        This function active a while loop with a time.sleep(2) until the game start.
        '''
        waitingGame = True
        while waitingGame:
            if game is True or game is False:
                print('Game cannot be a boolean!')
                break
            if self._detectGame(game) is True:
                waitingGame = False
                break
            time.sleep(2)
    def request_api(self, url):
        '''
        :param str url: API URL, must be in json format.
        
        Load HTTP GET with requests and take the information converting the body in json.
        '''
        req = r.get(url)
        if http_errors["error-response"] in str(req):
            print("Error in API:", http_errors["error-response"])
            exit()
        elif http_errors["failed-response"] in str(req):
            __http_response__("An error ocurred loading API:", http_errors["failed-response"])
        elif http_errors["rejected-response"] in str(req):
            __http_response__("The server has recieved the request but refuses to authorize it:", http_errors["rejected-response"])
        elif http_errors["timeout-response"] in str(req):
            __http_response__("Timeout:", http_errors["timeout-response"])
        elif http_errors["badgateway-response"] in str(req):
            __http_response__("Server error response code indicates that the server, while acting as a gateway or proxy, received an invalid response from the upstream server:", http_errors["badgateway-response"])
        elif http_errors["unauthorize-response"] in str(req):
            __http_response__("Unauthorize:", http_errors["unauthorize-response"])
        elif http_errors["notfound-response"] in str(req):
            __http_response__("HTTP Not found:", http_errors["notfound-response"])
        data = req.json()
        return data
    def _checkGameIsClosed(self, game):
        '''
        :param str game: Game
        
        Returns a boolean when the game is closed.
        '''
        if game is True or game is False:
            print('Game cannot be a boolean!')
            exit()
        if self._detectGame(game) is False:
            return True
        else:
            pass
        
    def destroy(self, rpc, text=""):
        '''
        :param str text: If want put error handler.
        :param pypresence.Presence rpc: RCP class for close the rcp and destroy the program.
        Destroy RPC and Program when something happen.
        '''
        if text=="":
            pass
        else:
            print(text)
            time.sleep(2)
        rpc.close()
        exit()


'''
Python RPCLayout for make more easier your code.
By: neokeee & etherlesss
'''

from .RPCLayout import RPCLayout

__title__ = "RPCLayout"
__author__ = "neokeee"
__copyright__ = "Copyright 2022 neokeee"
__license__ = 'MIT'
__version__ = '2.1.3'


import platform

class __test__:
    def callback_os(functionA):
        '''
        :param def void:
        Detect operating system and do a function
        '''
        if platform.system() == "Windows":
            functionA()
        elif platform.system() == "Darwin":
            functionA()
    


# Repositories uses in this example:
#   https://github.com/neopkr/RotMGRPC - by neokeee & etherlesss
#   https://github.com/qwertyquerty/pypresence - by qwertyquerty and +


# TEST LAYOUT

# Connecting rpc from pypresence
# https://github.com/qwertyquerty/pypresence


from pypresence import Presence
from RPCLayout import RPCLayout
import time

clientID = "955142324378296371" # replace with your client_id
rpc = Presence(clientID)
rpc.connect() # connecting rpc

# I will make a simple Realm of the Mad God Exalt RPC with an API - https://github.com/neopkr/RotMGRPC (LICENSE: MIT)
# URL API-KEY
url = "https://nightfirec.at/realmeye-api/?player=Neopkr&filter=player+characters+class+fame+rank"
layout = RPCLayout() # init class

''' Check if the game is open, i will comment this because you need the game and bla bla bla.
game_win32 = layout._detectGame('RotMG Exalt.exe') # Do exact for another OS
game_macOS = layout._detectGame('RotMGExalt') # _detectGame returns a boolean, true if game is open, false is game is closed
if game_win32 is True or game_macOS is True:
    print("RPC Connected!")
    pass
else:
    print("Game is not open!") # Here you have a choice to wait at user open the game. Using _waitingForOpen function.
    # for 2 OS, use platform
    if platform.system() == "Windows":
        _waitingForGame(game_win32)
    else:
        _waitingForGame(game_macOS)
'''
while True:
    data = layout.request_api(url) # init request, with this you have all ready to start.
    # Extracting data.json from api, thanks etherless.
    player_name = data['player']
    starQty = data['rank']
    charactersList = data['characters']
    playingAs = charactersList[0]
    
    # update made by etherless in https://github.com/neopkr/RotMGRPC
    rpc.update(
        state = f"Playing as: {playingAs['class']}",
        details = f"IGN: {player_name}",
        start = str(layout.timestamp()), # Use timestamp for start in 00:00 seconds
        large_image = "image_large", # Image imported in discord developer app
        large_text = 'Large Image Text!', # When you put the cursor over the large_image appears this text.
        small_image = 'star', # small_image, icon.
        small_text = str(starQty) # and the same at large_text. 
    ) # init update event from pypresence.Presence.update()
    
    # And that is, you're ready for use it.
    
    time.sleep(60) # while loop in 60 seconds. (ideal 5 seconds)


import setuptools

with open('README.md', "r", encoding="utf-8") as fh:
    long_description = fh.read()
    
setuptools.setup(
    name="RPCLayout",
    version="2.1.3",
    author="neokeee",
    author_email="neopkr90@gmail.com",
    description="Simple template API for Discord rich presence!",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/neopkr/RPCLayout",
    project_urls={
        "Bug Tracker": "https://github.com/neopkr/RPCLayout/issues"
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=['RPCLayout'],
    include_package_data=True,
    python_requires=">=3.5",
    platforms=['Windows', "OSX"],
    zip_safe=True,
    license="MIT"
)